@@ -99,6 +99,7 @@ applet_back_change (PanelApplet *a,
         	}
 		break;
 	}
+	gtk_rc_style_unref (rc_style);
 	return;
 }
 
@@ -194,7 +195,9 @@ timer_cb (EyesApplet *eyes_applet)
 }
 
 static void
-about_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
+about_cb (BonoboUIComponent *uic,
+	  EyesApplet        *eyes_applet,
+	  const gchar       *verbname)
 {
 	static GtkWidget *about = NULL;
 	GdkPixbuf	 *pixbuf;
@@ -212,9 +215,8 @@ about_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
 
 	const gchar *translator_credits = _("translator_credits");
 
-	if (about != NULL) {
-		gdk_window_show  (about->window);
-		gdk_window_raise (about->window);
+	if (about) {
+		gtk_window_present (GTK_WINDOW (about));
 		return;
 	}
         
@@ -240,8 +242,6 @@ about_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
 		gdk_pixbuf_unref (pixbuf);
 			
 	gtk_window_set_wmclass (GTK_WINDOW (about), "geyes", "Geyes");
-	gnome_window_icon_set_from_file (GTK_WINDOW (about),
-					 GNOME_ICONDIR"/gnome-eyes.png");	
 	g_signal_connect (about, "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
 			  &about);
@@ -355,19 +355,25 @@ destroy_cb (GtkObject *object, EyesApplet *eyes_applet)
 }
 
 static void
-help_cb (BonoboUIComponent *uic, gpointer user_data, const gchar *verbname)
+help_cb (BonoboUIComponent *uic,
+	 EyesApplet        *eyes_applet,
+	 const char        *verbname)
 {
-#ifdef FIXME
-	static GnomeHelpMenuEntry help_entry = { "geyes_applet", "index.html"};
-	gnome_help_display(NULL, &help_entry);
-#endif
+	GError *error = NULL;
+	gnome_help_display("geyes",NULL,&error);
+
+	if (error) {
+		g_warning ("help error: %s\n", error->message);
+		g_error_free (error);
+		error = NULL;
+	}
 }
 
 
 static const BonoboUIVerb geyes_applet_menu_verbs [] = {
-        BONOBO_UI_VERB ("Props", properties_cb),
-        BONOBO_UI_VERB ("Help", help_cb),
-        BONOBO_UI_VERB ("About", about_cb),
+        BONOBO_UI_UNSAFE_VERB ("Props", properties_cb),
+        BONOBO_UI_UNSAFE_VERB ("Help", help_cb),
+        BONOBO_UI_UNSAFE_VERB ("About", about_cb),
 
         BONOBO_UI_VERB_END
 };
@@ -391,7 +397,9 @@ static gboolean
 geyes_applet_fill (PanelApplet *applet)
 {
 	EyesApplet *eyes_applet;
-
+	
+	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-eyes.png");
+	
         eyes_applet = create_eyes (applet);
 
 	panel_applet_add_preferences (applet, "/schemas/apps/geyes/prefs", NULL);
@@ -444,7 +452,7 @@ geyes_applet_factory (PanelApplet *applet,
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_GeyesApplet_Factory",
 			     PANEL_TYPE_APPLET,
-			     "Geyes Applet",
+			     "geyes",
 			     "0",
 			     geyes_applet_factory,
 			     NULL)
