@@ -124,13 +124,13 @@ static const BonoboUIVerb applet_menu_verbs[] = {
 static const char applet_menu_xml[] =
 	"<popup name=\"button3\">\n"
 	"   <placeholder name=\"ChildMerge\"/>\n"
-	"   <menuitem name=\"Add\" verb=\"Add\" _label=\"Add...\"\n"
+	"   <menuitem name=\"Add\" verb=\"Add\" _label=\"_Add...\"\n"
 	"             pixtype=\"stock\" pixname=\"gtk-add\"/>\n"
-	"   <menuitem name=\"Properties Item\" verb=\"Properties\" _label=\"Preferences...\"\n"
+	"   <menuitem name=\"Properties Item\" verb=\"Properties\" _label=\"_Preferences...\"\n"
 	"             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
-	"   <menuitem name=\"Help Item\" verb=\"Help\" _label=\"Help\"\n"
+	"   <menuitem name=\"Help Item\" verb=\"Help\" _label=\"_Help\"\n"
 	"             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
-	"   <menuitem name=\"About Item\" verb=\"About\" _label=\"About ...\"\n"
+	"   <menuitem name=\"About Item\" verb=\"About\" _label=\"_About ...\"\n"
 	"             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
 	"</popup>\n";
 
@@ -273,6 +273,7 @@ panel_menu_background_restore (PanelMenu *panel_menu)
 
 	rc_style = gtk_rc_style_new ();
 	gtk_widget_modify_style (panel_menu->menubar, rc_style);
+	gtk_rc_style_unref (rc_style);
 }
 
 static gboolean
@@ -653,8 +654,9 @@ applet_help_cb (BonoboUIComponent *uic, PanelMenu *panel_menu,
 }
 
 static void
-applet_about_cb (BonoboUIComponent *uic, PanelMenu *panel_menu,
-		 const gchar *verbname)
+applet_about_cb (BonoboUIComponent *uic,
+		 PanelMenu         *panel_menu,
+		 const char        *verbname)
 {
 	static GtkWidget *about = NULL;
 	GdkPixbuf *pixbuf = NULL;
@@ -671,8 +673,7 @@ applet_about_cb (BonoboUIComponent *uic, PanelMenu *panel_menu,
 
 	const gchar *translator_credits = _("translator_credits");
 
-	if (about != NULL) {
-		gtk_widget_show (about);
+	if (about) {
 		gtk_window_present (GTK_WINDOW (about));
 		return;
 	}
